@@ -25,7 +25,7 @@ use crate::protos::*;
 use crate::query_planner::{PreFilterRequest, PreFilterResponse};
 use crate::IndexFiles;
 
-pub type TextsReaderPointer = Arc<dyn FieldReader>;
+pub type TextsReaderPointer = Arc<RwLock<dyn FieldReader>>;
 pub type TextsWriterPointer = Arc<RwLock<dyn FieldWriter>>;
 pub type ProtosRequest = DocumentSearchRequest;
 pub type ProtosResponse = DocumentSearchResponse;
