@@ -17,9 +17,13 @@
 // You should have received a copy of the GNU Affero General Public License
 // along with this program. If not, see <http://www.gnu.org/licenses/>.
 //
+
+use std::collections::HashSet;
 use std::path::PathBuf;
 use std::sync::{Arc, RwLock};
 
+use uuid::Uuid;
+
 use crate::metrics::vectors::MergeSource;
 use crate::prelude::*;
 use crate::protos::*;
@@ -33,9 +37,13 @@ pub type ProtosRequest = VectorSearchRequest;
 pub type ProtosResponse = VectorSearchResponse;
 
 #[derive(Debug, Clone, Copy)]
-pub struct MergeContext {
+pub struct MergeParameters {
     pub max_nodes_in_merge: usize,
     pub segments_before_merge: usize,
+}
+
+pub struct MergeContext {
+    pub parameters: MergeParameters,
     pub source: MergeSource,
 }
 
@@ -45,6 +53,7 @@ pub struct VectorConfig {
     pub path: PathBuf,
     pub vectorset: PathBuf,
     pub channel: Channel,
+    pub shard_id: String,
 }
 
 // In an ideal world this should be part of the actual request, but since
@@ -60,6 +69,17 @@ pub struct MergeMetrics {
     pub left: usize,
 }
 
+pub trait MergeResults {
+    fn inputs(&self) -> &HashSet<Uuid>;
+    fn output(&self) -> Uuid;
+    fn record_metrics(&self, source: MergeSource);
+    fn get_metrics(&self) -> MergeMetrics;
+}
+
+pub trait MergeRunner {
+    fn run(&mut self) -> NodeResult<Box<dyn MergeResults>>;
+}
+
 pub trait VectorReader: std::fmt::Debug + Send + Sync {
     fn search(&self, request: &ProtosRequest, context: &VectorsContext) -> NodeResult<ProtosResponse>;
     fn stored_ids(&self) -> NodeResult<Vec<String>>;
@@ -74,7 +94,8 @@ pub trait VectorWriter: std::fmt::Debug + Send + Sync {
     fn get_index_files(&self, ignored_segment_ids: &[String]) -> NodeResult<IndexFiles>;
     fn list_vectorsets(&self) -> NodeResult<Vec<String>>;
 
-    fn merge(&mut self, context: MergeContext) -> NodeResult<MergeMetrics>;
+    fn prepare_merge(&self, parameters: MergeParameters) -> NodeResult<Option<Box<dyn MergeRunner>>>;
+    fn record_merge(&mut self, merge_result: Box<dyn MergeResults>, source: MergeSource) -> NodeResult<MergeMetrics>;
     fn set_resource(&mut self, resource: &Resource) -> NodeResult<()>;
     fn delete_resource(&mut self, resource_id: &ResourceId) -> NodeResult<()>;
     fn garbage_collection(&mut self) -> NodeResult<()>;
