@@ -23,9 +23,9 @@ use crate::analytics::sync::send_analytics_event;
 use crate::grpc::collect_garbage::{garbage_collection_loop, GCParameters};
 use crate::merge::{global_merger, MergePriority, MergeRequest, MergeWaiter};
 use crate::settings::Settings;
+use crate::shards::cache::ShardWriterCache;
 use crate::shards::errors::ShardNotFoundError;
 use crate::shards::metadata::ShardMetadata;
-use crate::shards::providers::shard_cache::ShardWriterCache;
 use crate::shards::writer::ShardWriter;
 use crate::telemetry::run_with_telemetry;
 use crate::utils::{get_primary_node_id, list_shards, read_host_key};
@@ -194,7 +194,7 @@ impl NodeWriter for NodeWriterGRPCDriver {
         let write_task = || {
             run_with_telemetry(info, move || {
                 let shard = obtain_shard(shards, shard_id_clone)?;
-                shard.set_resource(&resource).and_then(|()| shard.get_opstatus())
+                shard.set_resource(resource).and_then(|()| shard.get_opstatus())
             })
         };
         let status = tokio::task::spawn_blocking(write_task)
@@ -343,7 +343,7 @@ impl NodeWriter for NodeWriterGRPCDriver {
     }
 
     async fn get_metadata(&self, _request: Request<EmptyQuery>) -> Result<Response<NodeMetadata>, Status> {
-        let settings = &self.settings.clone();
+        let settings = &self.settings;
         let mut total_disk = 0;
         let mut available_disk = 0;
 
@@ -353,8 +353,8 @@ impl NodeWriter for NodeWriterGRPCDriver {
         }
         Ok(tonic::Response::new(NodeMetadata {
             shard_count: list_shards(settings.shards_path()).await.len().try_into().unwrap(),
-            node_id: read_host_key(settings.host_key_path()).unwrap().to_string(),
-            primary_node_id: get_primary_node_id(settings.data_path()),
+            node_id: read_host_key(&settings.host_key_path).unwrap().to_string(),
+            primary_node_id: get_primary_node_id(&settings.data_path),
             available_disk,
             total_disk,
             ..Default::default()
