@@ -39,6 +39,29 @@ use crate::telemetry::run_with_telemetry;
 
 pub struct BlockingToken<'a>(MutexGuard<'a, ()>);
 
+const MAX_LABEL_LENGTH: usize = 32768; // Tantivy max is 2^16 - 4
+
+fn remove_invalid_labels(resource: &mut Resource) {
+    resource.labels.retain(|l| {
+        if l.len() > MAX_LABEL_LENGTH {
+            warn!("Label length ({}) longer than maximum, it will not be indexed", l.len());
+            false
+        } else {
+            true
+        }
+    });
+    for text in resource.texts.values_mut() {
+        text.labels.retain(|l| {
+            if l.len() > MAX_LABEL_LENGTH {
+                warn!("Label length ({}) longer than maximum, it will not be indexed", l.len());
+                false
+            } else {
+                true
+            }
+        });
+    }
+}
+
 #[derive(Debug)]
 pub struct ShardWriter {
     pub metadata: Arc<ShardMetadata>,
@@ -168,6 +191,7 @@ impl ShardWriter {
             path: path.join(VECTORS_DIR),
             vectorset: path.join(VECTORSET_DIR),
             channel,
+            shard_id: metadata.id(),
         };
         let rsc = RelationConfig {
             path: path.join(RELATIONS_DIR),
@@ -199,6 +223,7 @@ impl ShardWriter {
             path: path.join(VECTORS_DIR),
             vectorset: path.join(VECTORSET_DIR),
             channel,
+            shard_id: metadata.id(),
         };
         let rsc = RelationConfig {
             path: path.join(RELATIONS_DIR),
@@ -210,9 +235,11 @@ impl ShardWriter {
 
     #[measure(actor = "shard", metric = "set_resource")]
     #[tracing::instrument(skip_all)]
-    pub fn set_resource(&self, resource: &Resource) -> NodeResult<()> {
+    pub fn set_resource(&self, mut resource: Resource) -> NodeResult<()> {
         let span = tracing::Span::current();
 
+        remove_invalid_labels(&mut resource);
+
         let text_writer_service = self.text_writer.clone();
         let field_resource = resource.clone();
         let text_task = move || {
@@ -267,7 +294,7 @@ impl ShardWriter {
         let mut vector_result = Ok(());
         let mut relation_result = Ok(());
 
-        let _lock = self.write_lock.lock().expect("Poisoned write lock");
+        let _lock = self.write_lock.lock().unwrap_or_else(|e| e.into_inner());
         thread::scope(|s| {
             s.spawn(|_| text_result = text_task());
             s.spawn(|_| paragraph_result = paragraph_task());
@@ -327,7 +354,7 @@ impl ShardWriter {
         let mut vector_result = Ok(());
         let mut relation_result = Ok(());
 
-        let _lock = self.write_lock.lock().expect("Poisoned write lock");
+        let _lock = self.write_lock.lock().unwrap_or_else(|e| e.into_inner());
         thread::scope(|s| {
             s.spawn(|_| text_result = text_task());
             s.spawn(|_| paragraph_result = paragraph_task());
@@ -450,10 +477,18 @@ impl ShardWriter {
 
     #[tracing::instrument(skip_all)]
     pub fn merge(&self, context: MergeContext) -> NodeResult<MergeMetrics> {
-        let result = write_rw_lock(&self.vector_writer).merge(context);
+        let runner = read_rw_lock(&self.vector_writer).prepare_merge(context.parameters)?;
+        let Some(mut runner) = runner else {
+            return Ok(MergeMetrics {
+                merged: 0,
+                left: 0,
+            });
+        };
+        let merge_result = runner.run()?;
+        let metrics = write_rw_lock(&self.vector_writer).record_merge(merge_result, context.source)?;
         self.metadata.new_generation_id();
 
-        result
+        Ok(metrics)
     }
 
     /// This must be used only by replication and should be
@@ -464,7 +499,7 @@ impl ShardWriter {
     }
 
     pub async fn block_shard(&self) -> BlockingToken {
-        let mutex_guard = self.write_lock.lock().expect("Poisoned write lock");
+        let mutex_guard = self.write_lock.lock().unwrap_or_else(|e| e.into_inner());
         BlockingToken(mutex_guard)
     }
 
@@ -484,7 +519,7 @@ impl ShardWriter {
         ignored_segement_ids: &HashMap<String, Vec<String>>,
     ) -> NodeResult<Vec<(PathBuf, IndexFiles)>> {
         let mut files = Vec::new();
-        let _lock = self.write_lock.lock().expect("Poisoned write lock"); // need to make sure more writes don't happen while we are reading
+        let _lock = self.write_lock.lock().unwrap_or_else(|e| e.into_inner()); // need to make sure more writes don't happen while we are reading
         let paragraph_files = read_rw_lock(&self.paragraph_writer)
             .get_index_files(ignored_segement_ids.get("paragraph").unwrap_or(&Vec::new()))?;
         let text_files =
