@@ -19,12 +19,14 @@
 //
 
 #![allow(clippy::derive_partial_eq_without_eq)]
+#![allow(clippy::large_enum_variant)]
 
 pub mod fdbwriter;
 pub mod knowledgebox;
 pub mod nodereader;
 pub mod noderesources;
 pub mod nodewriter;
+pub mod replication;
 pub mod resources;
 pub mod utils;
 
