@@ -24,6 +24,7 @@ use std::time::Instant;
 use nucliadb_core::prelude::*;
 use nucliadb_core::protos::resource::ResourceStatus;
 use nucliadb_core::protos::{Resource, ResourceId};
+use nucliadb_core::relations::*;
 use nucliadb_core::tracing::{self, *};
 use nucliadb_core::{IndexFiles, RawReplicaState};
 use nucliadb_procs::measure;
@@ -55,15 +56,13 @@ impl RelationsWriterService {
     }
 }
 
-impl RelationWriter for RelationsWriterService {}
-
 impl std::fmt::Debug for RelationsWriterService {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         f.debug_struct("RelationWriterService").finish()
     }
 }
 
-impl WriterChild for RelationsWriterService {
+impl RelationsWriter for RelationsWriterService {
     #[measure(actor = "relations", metric = "count")]
     #[tracing::instrument(skip_all)]
     fn count(&self) -> NodeResult<usize> {
@@ -72,10 +71,7 @@ impl WriterChild for RelationsWriterService {
         let v = time.elapsed().as_millis();
         debug!("Count starting at {v} ms");
 
-        let count = self
-            .index
-            .start_reading()
-            .and_then(|reader| reader.no_nodes())?;
+        let count = self.index.start_reading().and_then(|reader| reader.no_nodes())?;
         let v = time.elapsed().as_millis();
         debug!("Ending at {v} ms");
 
@@ -109,10 +105,7 @@ impl WriterChild for RelationsWriterService {
             let v = time.elapsed().as_millis();
             debug!("{id:?} - Populating the graph: starts {v} ms");
 
-            let iter = resource
-                .relations
-                .iter()
-                .filter(|rel| rel.to.is_some() || rel.source.is_some());
+            let iter = resource.relations.iter().filter(|rel| rel.to.is_some() || rel.source.is_some());
             let mut writer = self.index.start_writing()?;
             for rel in iter {
                 let edge = relation_type_parsing(rel.relation(), &rel.relation_label);
@@ -125,11 +118,7 @@ impl WriterChild for RelationsWriterService {
                     from_type.0.to_string(),
                     from_type.1.map(|s| s.to_string()),
                 );
-                let to = IoNode::system_node(
-                    to.value.clone(),
-                    to_type.0.to_string(),
-                    to_type.1.map(|s| s.to_string()),
-                );
+                let to = IoNode::system_node(to.value.clone(), to_type.0.to_string(), to_type.1.map(|s| s.to_string()));
                 let edge = IoEdge::new(edge.0.to_string(), edge.1.map(|s| s.to_string()));
                 let metadata = rel.metadata.clone().map(IoEdgeMetadata::from);
                 writer.connect(&self.wmode, &from, &to, &edge, metadata.as_ref())?;
@@ -183,7 +172,10 @@ impl RelationsWriterService {
         } else {
             std::fs::create_dir(path)?;
             let (index, wmode) = Index::new_writer(path)?;
-            Ok(RelationsWriterService { index, wmode })
+            Ok(RelationsWriterService {
+                index,
+                wmode,
+            })
         }
     }
 
@@ -193,7 +185,10 @@ impl RelationsWriterService {
             Err(node_error!("Shard does not exist".to_string()))
         } else {
             let (index, wmode) = Index::new_writer(path)?;
-            Ok(RelationsWriterService { index, wmode })
+            Ok(RelationsWriterService {
+                index,
+                wmode,
+            })
         }
     }
 }
