@@ -20,9 +20,11 @@
 
 mod common;
 
-use nucliadb_core::prelude::*;
 use nucliadb_core::protos::{DocumentSearchRequest, Faceted, Filter};
+use nucliadb_core::query_planner::{PreFilterRequest, ValidFieldCollector};
+use nucliadb_core::texts::*;
 use nucliadb_texts::reader::TextReaderService;
+use std::collections::HashSet;
 
 #[test]
 fn test_search_queries() {
@@ -74,6 +76,64 @@ fn test_search_queries() {
     // query(&reader, "shou", 1);
 }
 
+#[test]
+fn test_prefilter_all_search() {
+    let reader = common::test_reader();
+    let request = PreFilterRequest {
+        security: None,
+        formula: None,
+        timestamp_filters: vec![],
+    };
+    let response = reader.prefilter(&request).unwrap();
+    assert!(matches!(response.valid_fields, ValidFieldCollector::All));
+}
+
+#[test]
+fn test_prefilter_not_search() {
+    let reader = common::test_reader();
+
+    let context = nucliadb_core::query_language::QueryContext {
+        field_labels: HashSet::from(["/l/mylabel".to_string()]),
+        paragraph_labels: HashSet::with_capacity(0),
+    };
+    let query = "{ \"not\": { \"literal\": \"/l/mylabel\" } }";
+    let expression = nucliadb_core::query_language::translate(query, &context).unwrap();
+    let request = PreFilterRequest {
+        security: None,
+        timestamp_filters: vec![],
+        formula: expression.prefilter_query,
+    };
+    println!("expression: {:?}", request.formula);
+    let response = reader.prefilter(&request).unwrap();
+    let valid_fields = &response.valid_fields;
+    let ValidFieldCollector::Some(fields) = valid_fields else {
+        panic!("Response is not on the right variant {valid_fields:?}");
+    };
+    assert_eq!(fields.len(), 1);
+}
+
+#[test]
+fn test_prefilter_search() {
+    let reader = common::test_reader();
+
+    let context = nucliadb_core::query_language::QueryContext {
+        field_labels: HashSet::from(["/l/mylabel".to_string()]),
+        paragraph_labels: HashSet::with_capacity(0),
+    };
+    let query = "{ \"literal\": \"/l/mylabel\" }";
+    let expression = nucliadb_core::query_language::translate(query, &context).unwrap();
+    let request = PreFilterRequest {
+        security: None,
+        formula: expression.prefilter_query,
+        timestamp_filters: vec![],
+    };
+    let response = reader.prefilter(&request).unwrap();
+    let ValidFieldCollector::Some(fields) = response.valid_fields else {
+        panic!("Response is not on the right variant");
+    };
+    assert_eq!(fields.len(), 1);
+}
+
 #[test]
 fn test_filtered_search() {
     fn query(reader: &TextReaderService, query: impl Into<String>, filter: Filter, expected: i32) {
@@ -88,8 +148,8 @@ fn test_filtered_search() {
         };
 
         let response = reader.search(&request).unwrap();
-        assert_eq!(response.total, expected, "Failed query: '{}'", query);
 
+        assert_eq!(response.total, expected, "Failed query: '{}'", query);
         assert_eq!(response.total, response.results.len() as i32);
         assert!(!response.next_page);
     }
@@ -101,6 +161,7 @@ fn test_filtered_search() {
         "",
         Filter {
             field_labels: vec!["/l/mylabel".to_string()],
+            expression: "{ \"literal\": \"/l/mylabel\" }".to_string(),
             ..Default::default()
         },
         1,
@@ -110,6 +171,7 @@ fn test_filtered_search() {
         "",
         Filter {
             field_labels: vec!["/e/myentity".to_string()],
+            expression: "{ \"literal\": \"/e/myentity\" }".to_string(),
             ..Default::default()
         },
         1,
@@ -119,6 +181,7 @@ fn test_filtered_search() {
         "",
         Filter {
             field_labels: vec!["/l/fakelabel".to_string()],
+            expression: "{ \"literal\": \"/l/fakelabel\" }".to_string(),
             ..Default::default()
         },
         0,
@@ -158,11 +221,7 @@ fn test_faceted_search() {
         };
         let response = reader.search(&request).unwrap();
         println!("Response: {response:#?}");
-        assert_eq!(
-            response.total, expected,
-            "Failed faceted query: '{}'. With facets: {:?}",
-            query, facets
-        );
+        assert_eq!(response.total, expected, "Failed faceted query: '{}'. With facets: {:?}", query, facets);
 
         assert_eq!(response.total, response.results.len() as i32);
         assert!(!response.next_page);
