@@ -17,9 +17,11 @@
 // You should have received a copy of the GNU Affero General Public License
 // along with this program. If not, see <http://www.gnu.org/licenses/>.
 
-use std::fmt::Debug;
-use std::time::Instant;
-
+use crate::data_point_provider::reader::Reader;
+use crate::data_point_provider::*;
+use crate::formula::{AtomClause, BooleanOperator, Clause, CompoundClause, Formula};
+use crate::indexset::ReaderSet;
+use crate::service::query_io;
 use nucliadb_core::prelude::*;
 use nucliadb_core::protos::prost::Message;
 use nucliadb_core::protos::{
@@ -28,11 +30,8 @@ use nucliadb_core::protos::{
 use nucliadb_core::tracing::{self, *};
 use nucliadb_core::vectors::*;
 use nucliadb_procs::measure;
-
-use crate::data_point_provider::*;
-use crate::formula::{AtomClause, BooleanOperator, Clause, CompoundClause, Formula};
-use crate::indexset::IndexSet;
-use crate::service::query_io;
+use std::fmt::Debug;
+use std::time::Instant;
 
 impl<'a> SearchRequest for (usize, &'a VectorSearchRequest, Formula) {
     fn with_duplicates(&self) -> bool {
@@ -53,8 +52,8 @@ impl<'a> SearchRequest for (usize, &'a VectorSearchRequest, Formula) {
 }
 
 pub struct VectorReaderService {
-    index: Index,
-    indexset: IndexSet,
+    index: Reader,
+    indexset: ReaderSet,
 }
 impl Debug for VectorReaderService {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
@@ -63,27 +62,26 @@ impl Debug for VectorReaderService {
 }
 
 impl VectorReader for VectorReaderService {
+    fn update(&mut self) -> NodeResult<()> {
+        self.index.update()?;
+        self.indexset.update()?;
+        Ok(())
+    }
+
     #[tracing::instrument(skip_all)]
     fn count(&self, vectorset: &str) -> NodeResult<usize> {
         if vectorset.is_empty() {
             debug!("Id for the vectorset is empty");
-            self.index_count(&self.index)
+            Ok(self.index.size())
+        } else if let Some(index) = self.indexset.get(vectorset)? {
+            debug!("Counting nodes for {vectorset}");
+            Ok(index.size())
         } else {
-            let indexet_slock = self.indexset.get_slock()?;
-            if let Some(index) = self.indexset.get(vectorset, &indexet_slock)? {
-                debug!("Counting nodes for {vectorset}");
-                self.index_count(&index)
-            } else {
-                debug!("There was not a set called {vectorset}");
-                Ok(0)
-            }
+            debug!("There was not a set called {vectorset}");
+            Ok(0)
         }
     }
 
-    fn update(&mut self) -> NodeResult<()> {
-        Ok(())
-    }
-
     #[measure(actor = "vectors", metric = "search")]
     #[tracing::instrument(skip_all)]
     fn search(&self, request: &ProtosRequest, context: &VectorsContext) -> NodeResult<ProtosResponse> {
@@ -94,8 +92,6 @@ impl VectorReader for VectorReaderService {
         let total_to_get = offset + request.result_per_page;
         let offset = offset as usize;
         let total_to_get = total_to_get as usize;
-        let indexet_slock = self.indexset.get_slock()?;
-        let index_slock = self.index.get_slock()?;
 
         let key_filters = request.key_filters.iter().cloned().map(AtomClause::key_prefix);
         let field_labels = request.field_labels.iter().cloned().map(AtomClause::label);
@@ -124,22 +120,16 @@ impl VectorReader for VectorReaderService {
 
         let result = if request.vector_set.is_empty() {
             debug!("{id:?} - No vectorset specified, searching in the main index");
-            self.index.search(&search_request, &index_slock)?
-        } else if let Some(index) = self.indexset.get(&request.vector_set, &indexet_slock)? {
+            self.index.search(&search_request)?
+        } else if let Some(index) = self.indexset.get(&request.vector_set)? {
             debug!("{id:?} - vectorset specified and found, searching on {}", request.vector_set);
-            let lock = index.get_slock()?;
-            index.search(&search_request, &lock)?
+            index.search(&search_request)?
         } else {
             debug!("{id:?} - A was vectorset specified, but not found. {} is not a vectorset", request.vector_set);
             vec![]
         };
         let v = time.elapsed().as_millis();
         debug!("{id:?} - Searching: ends at {v} ms");
-
-        std::mem::drop(indexet_slock);
-        std::mem::drop(index_slock);
-
-        let v = time.elapsed().as_millis();
         debug!("{id:?} - Creating results: starts at {v} ms");
 
         let documents = result
@@ -150,6 +140,7 @@ impl VectorReader for VectorReaderService {
             .flat_map(DocumentScored::try_from)
             .collect::<Vec<_>>();
         let v = time.elapsed().as_millis();
+
         debug!("{id:?} - Creating results: ends at {v} ms");
 
         let took = time.elapsed().as_secs_f64();
@@ -166,8 +157,7 @@ impl VectorReader for VectorReaderService {
     #[tracing::instrument(skip_all)]
     fn stored_ids(&self) -> NodeResult<Vec<String>> {
         let time = Instant::now();
-        let lock = self.index.get_slock().unwrap();
-        let result = self.index.get_keys(&lock)?;
+        let result = self.index.keys()?;
         let v = time.elapsed().as_millis();
         debug!("Ending at {v} ms");
 
@@ -208,18 +198,10 @@ impl VectorReaderService {
             return Err(node_error!("Invalid path {:?}", config.vectorset));
         }
         Ok(VectorReaderService {
-            index: Index::open(&config.path)?,
-            indexset: IndexSet::new(&config.vectorset)?,
+            index: Reader::open(&config.path)?,
+            indexset: ReaderSet::new(&config.vectorset)?,
         })
     }
-
-    #[measure(actor = "vectors", metric = "count")]
-    fn index_count(&self, index: &Index) -> NodeResult<usize> {
-        let lock = index.get_slock()?;
-        let no_nodes = index.no_nodes(&lock);
-        std::mem::drop(lock);
-        Ok(no_nodes)
-    }
 }
 
 #[cfg(test)]
@@ -295,8 +277,7 @@ mod tests {
         };
         // insert - delete - insert sequence
         let mut writer = VectorWriterService::start(&vsc).unwrap();
-        let res = writer.set_resource(&resource);
-        assert!(res.is_ok());
+        writer.set_resource(&resource).unwrap();
 
         let reader = VectorReaderService::start(&vsc).unwrap();
         let mut request = VectorSearchRequest {
