@@ -25,16 +25,14 @@ use std::sync::Arc;
 use crate::collect_garbage::{garbage_collection_loop, GCParameters};
 use crate::errors::{IndexNodeException, LoadShardError};
 use crate::RawProtos;
+use nucliadb_core::merge::MergerError;
 use nucliadb_core::protos::*;
 use nucliadb_core::Channel;
 use nucliadb_node::analytics::blocking::send_analytics_event;
 use nucliadb_node::analytics::payload::AnalyticsEvent;
+use nucliadb_node::cache::ShardWriterCache;
 use nucliadb_node::lifecycle;
-use nucliadb_node::merge::errors::MergerError;
-use nucliadb_node::settings::providers::env::EnvSettingsProvider;
-use nucliadb_node::settings::providers::SettingsProvider;
-use nucliadb_node::settings::Settings;
-use nucliadb_node::shards::cache::ShardWriterCache;
+use nucliadb_node::settings::load_settings;
 use nucliadb_node::shards::metadata::ShardMetadata;
 use nucliadb_node::shards::writer::ShardWriter;
 use prost::Message;
@@ -66,7 +64,7 @@ impl NodeWriter {
 impl NodeWriter {
     #[new]
     pub fn new() -> PyResult<Self> {
-        let settings: Settings = EnvSettingsProvider::generate_settings().unwrap();
+        let settings = load_settings().unwrap();
         let shard_cache = Arc::new(ShardWriterCache::new(settings.clone()));
         let shards_gc_loop_copy = Arc::clone(&shard_cache);
         let gc_parameters = GCParameters {
@@ -160,7 +158,7 @@ impl NodeWriter {
         let resource = Resource::decode(&mut Cursor::new(resource)).expect("Error decoding arguments");
         let shard_id = resource.shard_id.clone();
         let shard = self.obtain_shard(shard_id.clone())?;
-        let status = shard.set_resource(&resource).and_then(|()| shard.get_opstatus());
+        let status = shard.set_resource(resource).and_then(|()| shard.get_opstatus());
         match status {
             Ok(mut status) => {
                 status.status = 0;
